package ui;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import javax.swing.*;

import listener.CoordinateChangeEvent;
import listener.CoordinateListener;
import listener.LayerChangeEvent;
import listener.LayerListener;

import controller.BambooController;

import zuozuomu.img.*;

@SuppressWarnings("serial")
public class BambooPanel extends JPanel implements CoordinateListener,
		LayerListener {
	private BambooController controller;
	private ImageMaker imageMaker;
//	private Graphics graphics;
	private Graphics2D graphics;
	private BufferedImage mapImage;

	public BambooPanel(final BambooController controller) {
		this.controller = controller;
		makeMouseListener();
		imageMaker = new ImageMaker(controller);
		mapImage = new BufferedImage(controller.getPanelWidth(), controller
				.getPanelHeight(), BufferedImage.TYPE_INT_BGR);
//		graphics = mapImage.getGraphics();
		graphics = (Graphics2D) mapImage.getGraphics();
		graphics.setColor(Color.WHITE);
		graphics.fillRect(0, 0, controller.getPanelWidth(), controller
				.getPanelHeight());
		updateImage();
	}

	/* Ńplׂ̂Ă쐬 */
	public void paintComponent(final Graphics g) {
		super.paintComponent(g);
		g.drawImage(mapImage, 0, 0, null);

	}

	private void makeMouseListener() {
		addMouseListener(new MouseAdapter() {
			public void mousePressed(final MouseEvent e) {
				controller.mousePressed(e.getX(), e.getY());
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(final MouseEvent e) {
				controller.mouseDragged(e.getX(), e.getY());
			}
		});
	}

	private void updateImage() {
		imageMaker.makeMap(graphics);
		repaint();
	}

//	public void write(final Graphics g, final int left, final int right,
	public void write(final Graphics2D g, final int left, final int right,
			final int top, final int bottom) {
		imageMaker.write(g, left, right, top, bottom);
	}

	public void newMap(final int size) {
		// final int length = size * BambooView.sizeOfCell + 1;
		final int length = size * controller.getSizeOfCell() + 1;
		setPreferredSize(new Dimension(length, length));
		mapImage = new BufferedImage(length, length, BufferedImage.TYPE_INT_BGR);
//		graphics = mapImage.getGraphics();
		graphics = (Graphics2D) mapImage.getGraphics();
		graphics.setColor(Color.WHITE);
		graphics.fillRect(0, 0, length, length);
		updateImage();
	}

	public void coordinateChanged(final CoordinateChangeEvent e) {
		updateImage();
	}

	public void layerChanged(final LayerChangeEvent e) {
		updateImage();
	}
}
